#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#import gtk.glade
#import gtk
import threading

import gi

gi.require_version('Gtk','3.0')
from ctypes import *
import logging
import logging.config
import os
import pwd
import sys
import traceback

import pexpect
from gi.repository import Gdk, Gio, GLib, GObject, Gtk

#import gobject
from ui.ui_cliente import Ui_Cliente
import time
try:
  x11 = ctypes.cdll.LoadLibrary('/usr/lib/x86_64-linux-gnu/caca/libx11_plugin.so')
  #x11.XInitThreads()
except:
  print("Advertencia: falló en XInitThreads()")

DIRECTORIO = "/usr/share/pyshared/cga-asistencia-remota-cliente/"


def config_logging():
    user_path = os.path.join( os.path.expanduser("~"), 
                              ".cga-soporte-remoto/")
    if not os.path.exists(user_path):
        os.mkdir(user_path)
    
    my_defaults = {"home" :os.path.expanduser("~")}
    logfile = os.path.join( os.path.dirname(__file__), 
                            "./log.cfg")
    logging.config.fileConfig( logfile, 
                               defaults=my_defaults)
    root_handler = logging.getLogger("root")


class Scr_Cliente(GObject.GObject):
    
    __gsignals__ = { "conection-status"     : ( GObject.SignalFlags.RUN_FIRST,
                                                GObject.TYPE_NONE, 
                                                [   #estado de la conexion
                                                    GObject.TYPE_BOOLEAN,])
                    }
    
    def __init__(self):
        
        super(Scr_Cliente, self).__init__()
        
        self.user_name = str(pwd.getpwuid(os.getuid())[0])
        self.puerto_pasarela = 8888
        self.puerto_local = 22
        self.servidor_pasarela = "guadalinexedu.org"
        self.usuario = "usuario"
        self.password = "usuario"
                
        self.query_rsa = "Enter passphrase for key '/home/%s/.ssh/id_rsa':" \
                         %(self.user_name)
        self.query_dsa = "Enter passphrase for key '/home/%s/.ssh/id_dsa':" \
                         %(self.user_name)

    
    def start_conection( self, signal, puerto):
        t = threading.Thread(target=self.__do_connection, 
                             args=(puerto,))
        t.start()
    
    def __do_connection(self,puerto):
        logging.debug("Se inicia la conexion")
        command = "ssh -o \"ServerAliveInterval 15\" -p%s -C -R %s:localhost:%s %s@%s" \
                        %(self.puerto_pasarela,
                          puerto,
                          self.puerto_local,
                          self.usuario,
                          self.servidor_pasarela)
        logging.debug("Ejecutando comando: %s" %command)
        #Como python3 no se lleva muy bien con el multihilo hay que hacer que pare un poco, por eso
        #el time.sleep(1)
        time.sleep(1)
        self.conection = pexpect.spawn(command)
        connected = False
        while not connected:
            i = self.conection.expect(["Are you sure you want to continue connecting (yes/no)?",
                                       self.query_rsa,
                                       self.query_dsa,
                                       "%s@%s's password:" %(self.usuario,
                                                             self.servidor_pasarela), 
                                       pexpect.EOF, pexpect.TIMEOUT])
            if i == 0: 
                # yes/no
                logging.debug("Pide yes/no")
                self.conection.sendline("yes")
            elif i == 1: 
                # rsa
                logging.debug("Pide rsa")
                self.conection.sendline("\n")
            elif i == 2: 
                # dsa
                logging.debug("Pide dsa")
                self.conection.sendline("\n")
            elif i == 3: 
                # user password
                logging.debug("Login con usuario/contraseña")
                self.conection.sendline(self.password)
                connected = True
                break
            elif i == 4:
                logging.error("Error en la conexión ssh. Motivo: EOF")
                self.close_spawns()
                break
            elif i == 5:
                logging.debug("Error en la conexión ssh. Motivo: TIMEOUT")
                self.close_spawns()
                break
        logging.debug("Fin de la conexion")
        self.emit("conection-status", connected)
        
    def close_spawns(self, *args):
        if hasattr(self, "conection") and self.conection.isalive():
            result = self.conection.terminate()
            if not result:
                logging.debug("Se fuerza el cierre de la conexion")
                self.conection.terminate(force=True)
            else:
                logging.debug("Se cierra la conexion")
            self.emit("conection-status", False)
    
def main():
    config_logging()
    scr = Scr_Cliente()
    glade_file = DIRECTORIO +"ui/glade/cliente_v2.glade"
    ui = Ui_Cliente(glade_file)
    
    scr.connect("conection-status",  ui.connection_handler)
    ui.connect("port-inserted",      scr.start_conection)
    ui.connect("close-connections",  scr.close_spawns)
    ui.mostrar()
    
    logging.debug("En main:")
    Gtk.main()
    
    
if __name__ == "__main__":
    try:
        main()
    except Exception as e:
        print (str(e))
        os._exit(1)

