#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#import gobject
#import gtk.glade
import os
import logging
import threading
#import gtk
#gtk.gdk.threads_init()
import time
import gi
gi.require_version('Gtk','3.0')
from gi.repository import Gtk, Gio, GLib, GObject, Gdk

import sys
import ctypes
x11 = ctypes.cdll.LoadLibrary('/usr/lib/x86_64-linux-gnu/caca/libx11_plugin.so')
#x11.XInitThreads()

GObject.threads_init()
from ui.ui_info import Ventana_info

from ui.abstract_parent import Gtk_parent



DIRECTORIO = "/usr/share/pyshared/cga-asistencia-remota-cliente/"


## Interfaz gráfica de Cliente
class Ui_Cliente(GObject.GObject, Gtk_parent):
    
    __gsignals__ = { "port-inserted"   : (GObject.SignalFlags.RUN_FIRST, 
                                         GObject.TYPE_NONE, 
                                         [GObject.TYPE_INT,]),
                     "close-connections":   (GObject.SignalFlags.RUN_FIRST, 
                                             GObject.TYPE_NONE, 
                                             []),
                    }
    
    def __init__(self, glade_file):
        GObject.GObject.__init__(self)
        Gtk_parent.__init__(self, glade_file)
        
    def init_widgets(self):
        self.__conectado = False
        self.__entry_codigo = self.get_widget("ent_cod")
        #Establecemos la longitud máxima del puerto
        self.__entry_codigo.set_max_length(10)
        self.v_principal = self.get_widget("Vprincipal")
        self.v_principal.set_wmclass("Soporte remoto", "Soporte remoto")
        self.v_principal.set_icon_from_file("/usr/share/pixmaps/cga_asistencia_remota.svg")
        self.__fixed = self.get_widget("fixed1")
        self.__spinner = self.get_widget("spinner1")
        self.__b_desconectar = self.get_widget("bDesconectar")
        self.__b_desconectar.set_sensitive(self.__conectado)
        self.__b_conectar = self.get_widget("bConectar")
        self.__img_info = self.get_widget("img_info")
        self.__msg_info = self.get_widget("label_info")
        self.__entry_status = self.get_widget("label_status")       

    def connect_signals(self):
        self.v_principal.connect("delete-event", self.__cerrar)
        self.v_principal.connect("destroy-event", self.__cerrar)
        self.__b_desconectar.connect("clicked", self.emitir_desconectar)
        self.__b_conectar.connect("clicked", self.__aceptar_clicked)
        self.__entry_codigo.connect("focus-in-event", 
                                    self.__on_focus_handler)

        
    def __on_focus_handler (self, widget, event):
        self.info_set_visibility(False)
        
    def info_set_visibility(self, visible):
        self.__img_info.set_visible(visible)
        self.__msg_info.set_visible(visible)
        
    def set_entry_codigo(self, codigo):
        self.__entry_codigo.set_text(codigo)
    
    def mostrar(self):
        self.v_principal.show()
    
    def enable_port_entry(self):
        self.__entry_codigo.set_sensitive(True)
    
    def __aceptar_clicked(self, widget,data=None):
        print ("funcion aceptar_clicked")
        puerto_str = self.__entry_codigo.get_text().lower()
        logging.debug("Chequeando puerto en cadena %s" %puerto_str)
        if len(puerto_str) == 10 and self.__validate_string(puerto_str):
            puerto = int(puerto_str[2:-3])
            logging.debug("Se lanza la señal port-inserted:%s" %(puerto))
            self.emit("port-inserted", int(puerto))
            t = threading.Thread(target=self.__proceso_espera_start)
            t.start()
        else:
            logging.debug("Cadena incorrecta: %s" %(puerto_str))
            self.info_set_visibility(True)
    
    def __proceso_espera_start(self):
        GLib.timeout_add(250, self.__start_wait)
    def __proceso_espera_stop(self):
        GLib.timeout_add(250, self.__stop_wait)
     
        
    def __start_wait(self):
        self.__entry_codigo.set_sensitive(False)
        self.__set_sentisitive_buttons(False)
        self.__fixed.set_visible(False)
        logging.debug("Se inicia el spinner")
        self.__spinner.set_visible(True)
        self.__spinner.start()
        
    def __stop_wait(self):
        logging.debug("Se para el spinner")
        self.__spinner.stop()
        self.__set_sentisitive_buttons(True)
        self.__spinner.set_visible(False)
        self.__fixed.set_visible(True)
    
    def __validate_string(self, puerto_str):
        es_hexadecimal = self.is_hexadecimal(puerto_str)
        son_digitos = puerto_str[2:-3].isdigit()
        logging.debug("Es hexadecimal: %s y son_digitos %s"
                      %(es_hexadecimal, son_digitos)) 
        if es_hexadecimal and son_digitos:
            return True
        else:
            return False
    
    def is_hexadecimal(self, puerto):
        try:
            number = int(puerto,16)
        except ValueError:
            return False
        return True
    
    def __cerrar (self, widget,data=None):
        #cerramos la conexiones establecidas con pasarela
        #os.popen("ps fax|grep educandoenred.org|awk '/ / {print $1}' | xargs kill -9")
        self.emit("close-connections")
        self.v_principal.hide()
        Gtk.main_quit()
        
    def __set_sentisitive_buttons(self, sensitive):
        self.__b_conectar.set_sensitive(sensitive)
        self.__b_desconectar.set_sensitive(sensitive)
        
        
    def connection_handler(self, signal, state):
        # solo controlamos la señal cuando estabamos desconectados y pasamos
        # a conectar
        if not self.get_status():
            self.__proceso_espera_stop()
            #self.__stop_wait()
            self.set_status(state)
            if not state:
                glade_file = DIRECTORIO +"ui/glade/cliente_v2.glade"
                self.v_info = Ventana_info(glade_file)
                self.v_info.connect("on-close", self.on_close_v_info)
                self.v_principal.hide()
                self.v_info.show()
            
    def on_close_v_info(self,signal):
        self.v_principal.show()
        self.v_info = None
        
    def get_status(self):
        return self.__conectado
        
    def set_status(self, state):
        self.__conectado = state
        if state:
            # si hay conexion entonces...
            self.__b_conectar.set_sensitive(False)
            self.__b_desconectar.set_sensitive(True)
            self.__entry_status.set_text( "Conectado")
        else:
            # si no hay conexion entonces...
            self.__b_conectar.set_sensitive(True)
            self.__b_desconectar.set_sensitive(False)
            self.__entry_status.set_text( "Desconectado")
            self.enable_port_entry()
        
    def emitir_desconectar(self, *args):
        self.emit("close-connections")
        self.set_status(False)
        

